--
-- Rocket Launcher class
-- by CraigRock, John & Jozef
--

Rocket = {}

local Rocket_mt = Class(Rocket);

Rocket.allRockets = {};

function Rocket:new(rocketLauncher, filename, baseDir, x,y,z, dx,dy,dz, velocity, gravity)
    local self = {};
    setmetatable(self, Rocket_mt);

	self.calledRocketCollided = false;
	
    self.rocketLauncher = rocketLauncher;

    local i3dNode = Utils.loadSharedI3DFile(filename, baseDir);
    self.rocketNode = getChildAt(i3dNode, 0);
    link(getRootNode(), self.rocketNode);
    delete(i3dNode);

    setTranslation(self.rocketNode, x,y,z);

    setDirection(self.rocketNode, -dx,-dy,-dz, 0,1,0);
    
    self.posX = x;
    self.posY = y;
    self.posZ = z;
    self.velX = dx*velocity;
    self.velY = dy*velocity;
    self.velZ = dz*velocity;
    self.gravity = gravity;
    self.hadCollision = false;
    
    self.lifetime = 7500;

    self.exhaustParticleSystem = {};
    local rocketExhaustParticleSystemData = {};
    rocketExhaustParticleSystemData.nodeStr = "0>";
    rocketExhaustParticleSystemData.psFile = "exhaustParticleSystem.i3d";
    rocketExhaustParticleSystemData.posX, rocketExhaustParticleSystemData.posY, rocketExhaustParticleSystemData.posZ = 0;
    rocketExhaustParticleSystemData.rotX, rocketExhaustParticleSystemData.rotY, rocketExhaustParticleSystemData.rotZ = 0;
    Utils.loadParticleSystemFromData(rocketExhaustParticleSystemData, self.exhaustParticleSystem, getRootNode(), true, nil, baseDir);
    Utils.setEmittingState(self.exhaustParticleSystem, true);	
    for k, v in ipairs(self.exhaustParticleSystem) do
		link(self.rocketNode, getEmitterShape(getGeometry(v.shape)));
    end;	

    Rocket.allRockets[self.rocketNode] = self;

    return self;
end;

function Rocket:delete()
	
	g_currentMission:removeUpdateable(self);
    
	Rocket.allRockets[self.rocketNode] = nil;
    delete(self.rocketNode);
	Utils.deleteParticleSystem(self.exhaustParticleSystem);

	--print("delete");
	
end;

function Rocket:update(dt)

	--print("update");

    self.lifetime = self.lifetime - dt;
	
    if self.lifetime <= 0 then
		if not self.calledRocketCollided then
			self.rocketLauncher:onRocketCollided(self, self.posX, self.posY, self.posZ);
		end;
        self:delete();
        return;
    end;
    local velocity = Utils.vector3Length(self.velX, self.velY, self.velZ);
    raycastClosest(self.posX, self.posY, self.posZ, self.velX, self.velY, self.velZ, "raycastRocketCallback", velocity*dt, self, 32, true);
    if self.hadCollision and not self.calledRocketCollided then
        self.rocketLauncher:onRocketCollided(self, self.posX, self.posY, self.posZ);
        --self:delete();
		Utils.setEmittingState(self.exhaustParticleSystem, false);	
		setTranslation(self.rocketNode, 0, -1000, 0);
		self.calledRocketCollided = true;
        return;
    else
        -- update the position and velocity
        self.velY = self.velY - self.gravity*dt;
        self.posX = self.posX + self.velX*dt;
        self.posY = self.posY + self.velY*dt;
        self.posZ = self.posZ + self.velZ*dt;
        setTranslation(self.rocketNode, self.posX, self.posY, self.posZ);
        --setDirection(self.rocketNode, self.velX,self.velY,self.velZ, 0,1,0);
    end;
end;

function Rocket:raycastRocketCallback(transformId, x, y, z, distance, nx, ny, nz, subShapeIndex)
    if transformId ~= 0 then
        self.hadCollision = true;
    end;
    return false;
end;
